/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/*******************************************************************************
* File Name    : AR_LIB_WIRE.c
* Version      : Rev. 1.10
* Device(s)    : R5F104ML
* Tool-Chain   : CCRL
* Description  : This file implements Arduino library(Wire).
* Creation Date: 2021/01/01
* Modified Data: 2021/06/25
********************************************************************************/

/*******************************************************************************
Includes
********************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_timer.h"
#include "r_cg_serial.h"
#include "r_cg_userdefine.h"
#include "AR_LIB_WIRE.h"
#include "AR_WIRE_DEF.h"
#include <stdarg.h>
#include <stdio.h>

/*******************************************************************************
Pragma directive
********************************************************************************/
#ifdef   CS_ENV
#pragma interrupt r_IICA0_interrupt(vect=INTIICA0,bank=RB1,enable=true)
#endif



/*******************************************************************************
Global variables and functions
********************************************************************************/

volatile    uint8_t gp_tx_set;          /* transmit data set pointer            */
volatile    uint8_t gp_tx_get;          /* transmit data read pointer           */
volatile    uint8_t g_tx_buff[256];     /* I2C transmit data queue              */

volatile    uint8_t gp_rx_set;          /* received data set pointer            */
volatile    uint8_t gp_rx_get;          /* received data read pointer           */
volatile    uint8_t g_rx_buff[256];     /* I2C recived data queue               */

volatile    uint16_t g_rx_num;          /* remaining receive data number        */
volatile    uint8_t *tx_pointer;        /* transmit data pointer                */
volatile    uint8_t sladdr8;            /* 8-bit slave address                  */

volatile    uint8_t g_stop_flag;        /* stop issue flag 0:do not/1:issue     */

volatile    uint8_t g_status;           /* IICA0 status
    0x00 : BUS_FREE     :BUS FREE
    0x8F : BUS_ERROR    :BUS Error
    0x10 : GET_BUS      :get I2C bus
    0x20 : GET_BUS4TX   :get I2C bus to transmit
    0x30 : TX_MODE      :Transmit operation
    0x40 : TX_END       :Transmit end
    0x50 : GET_BUS4RX   :get bus to receive
    0x60 : RX_MODE      :Receive operation
    0x70 : RX_END       :Receive end
    0x81 : BUFF_OVER    :Buffer over
    0x82 : NO_SLAVE     :No slave
    0x83 : NO_ACK       :No ACK for data
*/

volatile    uint8_t g_erflag;           /* wire function result status
    0x00 :  success
    0x01 :  buffer over flow
    0x02 :  No ACK for slave address
    0x03 :  No ACK for data
    0x04 :  other error
    */


/*--------------------------------------------------------------

    definition of wire API functions

---------------------------------------------------------------*/

API_Wire Wire = {
    Wire_begin,                         /*start of wire function as master      */
    Wire_beginTransmission,             /* start transmit to slave              */
    Wire_writeb,                        /* data set to transmit buffer          */
    Wire_writec,                        /* data set to transmit buffer          */
    Wire_endTransmission,               /* end transmit to slave                */
    Wire_requestFrom,                   /* start receiving                      */
    Wire_requestFromS,                  /* start receiving                      */
    Wire_available,                     /* check receive buffer                 */
    Wire_read,                          /* read received data from buffer       */
};



/* start receive data from slave    */
void Wire_requestFromSub(uint8_t saddr7, uint16_t bytes);


void r_operation_end(void);             /* end to transfer, keep or lerease bus */
uint8_t r_start_condition(void);        /* isuue start condition                */
uint8_t r_stop_condition(void);         /* isuue stop condition                 */
/***********************************************************************
* Function Name: Wire_begin
* Description  : This function initialize Wire to use as master
* Arguments    : none
* Return Value : none
* Remark       : 
************************************************************************/
void Wire_begin(void)
{

    gp_tx_set = 0;                      /* initialize pointers for data queues  */
    gp_tx_get = 0;
    gp_rx_set = 0;
    gp_rx_get = 0;
    
    R_IICA0_Create();                   /* re_initialize IICA0                  */

}   /* end of function Wire_begin  */


/***********************************************************************
* Function Name: Wire_beginTransmission
* Description  : This function start write to slave
* Arguments    : saddr7: slave address of 7 bit
* Return Value : g_erflag
* Remark       : g_status = GET_BUS4TX(0x20) : success
*                           BUS_ERROR(0x8F)  : fail
************************************************************************/
uint8_t Wire_beginTransmission(uint8_t saddr7)
{

    g_erflag = 0x04;                    /* set dummy error flag                 */

    if ( RX_MODE == g_status )          /* if receive mode exit with error flag */
    {
        g_status = BUS_ERROR;
        return(g_erflag);
    }

    sladdr8     = (saddr7 << 1);        /* get 8bit slave address               */

    g_status = r_start_condition();     /* issue start condition                */

    if ( 0 == g_status )
    {
        gp_tx_set = 0;                  /* initialize pointers for data queues  */
        gp_tx_get = 0;

        g_status = GET_BUS4TX;          /* set status to "getting I2C bus"      */
        g_erflag = 0x00;                /* set success flag                     */

    }

    return(g_erflag);

}   /* end of function Wire_beginTransmission   */


/***********************************************************************
* Function Name: Wire_writec
* Description  : This function set a charactor to transmit buffer
* Arguments    : data
* Return Value : data number in transmit data queues
* Remark       : g_status = GET_BUS4TX(0x20) : success
*                           BUS_ERROR(0x8F)  : fail
*                           BUFF_OVER(0x81)  : fail
*               g_erflag  = 
************************************************************************/
uint8_t Wire_writec(uint8_t data)
{

    if ( 250 <= gp_tx_set )
    {
        g_status = BUFF_OVER;           /* set data over flag                   */
        g_erflag = 0x01;                /* set buffer over flow flag            */
        return(gp_tx_set);
    }

    g_erflag = 0x04;                    /* set dummy error flag                 */

    if ( GET_BUS4TX == g_status )
    {
        g_tx_buff[gp_tx_set] = data;    /* set transmit data in data queues     */
        ++gp_tx_set;                    /* move pointer to next address         */
        g_erflag = 0x00;                /* set success flag                     */
    }

    return(gp_tx_set);                  /* return data number in queues         */

}   /* end of function Wire_writec  */


/***********************************************************************
* Function Name: Wire_writec
* Description  : This function set a block data to transmit buffer
* Arguments    : data
* Return Value : data number in transmit data queues
* Remark       : g_status = GET_BUS4TX(0x20) : success
*                           BUS_ERROR(0x8F)  : fail
*                           BUFF_OVER(0x81)  : fail
************************************************************************/
uint8_t Wire_writeb(uint8_t *buff, uint8_t bytes)
{

    uint8_t loop;
    uint8_t *ptr;

    if ( 250 <= ( gp_tx_set + bytes ))
    {
        g_status = BUFF_OVER;           /* set data over flag                   */
        g_erflag = 0x01;                /* set buffer over flow flag            */
        return(gp_tx_set);
    }

    g_erflag = 0x04;                    /* set dummy error flag                 */

    if ( GET_BUS4TX == g_status )
    {
        ptr = buff;
        for ( loop = 0 ; loop < bytes ; loop++ )
        {
            g_tx_buff[gp_tx_set] = *ptr; /* set transmit data in data queues    */
            ++gp_tx_set;                /* move set pointer to next address     */
            ++ptr;                      /* move read pointer                    */
        }
        g_erflag = 0x00;                /* set success flag                     */
    }

    return(gp_tx_set);                  /* return data number in queues         */


}   /* end of function Wire_writeb  */


/***********************************************************************
* Function Name: Wire_endTransmission
* Description  : This function actually start write data to slave
* Arguments    : stop  : FALSE(0):do not issue STOP /TRUTH(1):issue STOP
* Return Value : g_erflag = 0x00 : success
*                           0x02 : NACK for slave address
*                           0x03 : NACK for transmit data
*                           0x04 : other error
* Remark       : g_status = 0x10 : success
*                           0x8F : fail
************************************************************************/
uint8_t Wire_endTransmission(uint8_t stop)
{

    if ( GET_BUS4TX != g_status )       /* check if transmit mode or not        */
    {
        g_status = BUS_ERROR;           /* set bus error flag                   */
        g_erflag = 0x04;
        return(g_erflag);               /* exit with error flag                 */
    }

    g_stop_flag = stop;                 /* set stop condition flag              */

    IICA0 = sladdr8;                    /* transmit slave address               */

    g_status = TX_MODE;                 /* set status to transmit mode          */


/*--------------------------------------------------
  transmission start and  wait for transmission end
--------------------------------------------------*/

    while ( TX_MODE == g_status )       /* wait for transmission end            */
    {
        NOP();
    }

    return(g_erflag);                   /* exit with result flag                */
    
}   /* end of function Wire_endTransmission   */


/***********************************************************************
* Function Name: Wire_requestFrombS
* Description  : This function start read from slave with STOP condition
* Arguments    : 
*               saddr7: slave address of 7 bit
*               bytes : receive data number
* Return Value : g_erflag = 0x00 : success
*                           0x01 : buffer over flow
* Remark       : g_status = 0x40 : success
*                           0x81 : data number error
*                           0x8F : fail
************************************************************************/
uint8_t Wire_requestFromS(uint8_t saddr7, uint16_t bytes)
{

    g_stop_flag = TRUTH;                /* set issue stop flag                  */
    Wire_requestFromSub(saddr7, bytes);
    return(g_erflag);

}   /* end of function Wire_requestFromS    */


/***********************************************************************
* Function Name: Wire_requestFrom
* Description  : This function start read datas from slave with STOP condition
* Arguments    : 
*               saddr7: slave address of 7 bit
*               bytes : receive data number
*               stop  : FALSE(0):do not issue STOP /TRUTH(1):issue STOP
* Return Value : g_erflag = 0x00 : success
*                           0x01 : buffer over flow
* Remark       : g_status = 0x40 : success
*                           0x81 : data number error
*                           0x8F : fail
************************************************************************/
uint8_t Wire_requestFrom(uint8_t saddr7, uint16_t bytes, uint8_t stop)
{

    g_stop_flag = stop;                 /* set issue stop flag                  */
    Wire_requestFromSub(saddr7, bytes);
    return(g_erflag);

}  /* end of function Wire_requestFrom  */


/***********************************************************************
* Function Name: Wire_requestFromSub
* Description  : This function issue start condition for read
* Arguments    : 
*               saddr7: slave address of 7 bit
*               bytes : receive data number
* Return Value : none
* Remark       : g_status = 0x50 : success
*                           0x81 : data number error
*                           0x84 : no data is requested
*                           0x8F : fail
*                g_erflag = 0x00 : success
*                           0x01 : buffer over flow
*                           0x04 : other error
************************************************************************/
void Wire_requestFromSub(uint8_t saddr7, uint16_t bytes)
{

    g_erflag = 0x00;                    /* clear error flag                     */

    if ( 255 <= bytes )
    {
        g_status = BUFF_OVER;           /* requested too much number of data    */
        g_erflag = 0x01;                /* set error status set                 */
        return;
    }
    if ( 0 == bytes )
    {
        g_status = NO_DATA;             /* no data is requested                 */
        g_erflag = 0x04;                /* set no data is requested error flag  */
        return;
    }

    if (( TX_MODE == g_status )||( RX_MODE == g_status) )
    {
        g_erflag = 0x04;                /* set error flag                       */
        return;                         /* exit with error flag                 */
    }

/*--------------------------------------------------
  end of parameter check  and start operation
--------------------------------------------------*/

    g_rx_num    = (uint8_t)bytes;       /* set recive data number               */
    gp_rx_set   = 0x00;                 /* initialize buffer pointer            */
    gp_rx_get   = 0x00;

    sladdr8     = (saddr7 << 1);        /* get 8bit slave address               */

    g_status = r_start_condition();     /* issue start condition                */

    if ( 0 == g_status )
    {

        g_status = GET_BUS4RX;
        IICA0 = ( sladdr8 | 0x01 );     /* transmit slave address for receive   */

    }

    /*
    transfer is done as back ground operation.
    g_status showes status of operation.
    g_erflag showes the error
    and Wire_availabe() can get received data number.
    */


}  /* end of function Wire_requestFromSub  */


/***********************************************************************
* Function Name: Wire_available
* Description  : This function return available data number in receive buffer
* Arguments    : none
* Return Value : data number in receive buffer
* Remark       : 
************************************************************************/
uint8_t Wire_available(void)
{

    uint8_t work;

    work = gp_rx_set - gp_rx_get;       /* get remaining data number            */ 

    return( work );                     /* return remaining data number         */

}   /* end of function Wire_available   */



/***********************************************************************
* Function Name: Wire_read
* Description  : This function return data at top of buffer
* Arguments    : none
* Return Value : data or 0x00(if there is no more data)
* Remark       : 
************************************************************************/
uint8_t Wire_read(void)
{

    uint8_t work;

    work = gp_rx_set - gp_rx_get;       /* get remaining data number            */ 

    if ( 0x00 != work )
    {
        work = g_rx_buff[gp_rx_get];    /* get data from buffer                 */
        ++gp_rx_get;                    /* move read pointer                    */
    }
    else
    {
        g_erflag = 0x01;                /* set error flag                       */
    }

    return( work );

}   /* end of function Wire_read    */


/***********************************************************************
* Function Name: r_IICA0_interrupt
* Description  : This function interrupt(INTIICA0) service routine.
* Arguments    : none
* Return Value : none
* Remark       : 
************************************************************************/
#ifdef  CS_ENV
static void __near r_IICA0_interrupt(void)
#endif
#ifdef  IAR_ENV
#pragma vector = INTIICA0_vect
__interrupt void r_IICA0_interrupt(void)
#endif
{

    if ( 0 == MSTS0)
    {                                           /* fail to get I2C bus          */
        LREL0 = 1;
        g_status = BUS_ERROR;                   /* set fail status              */
        g_erflag = 0x04;                        /* set error flag               */
        return;                                 /* exit from interrupt          */
    }

/*======================================================
  I2C master communication procedure
======================================================*/

    if ( 1 == STD0 )                            /* slave address transmit end   */
    {

/*----------------------------------------------
  slave address transmission end
----------------------------------------------*/

        if ( 1 == ACKD0 )
        {

/*======================================================
  ACK response for slave address and start data transfer
======================================================*/

            if ( 1 == TRC0 )
            {

/*--------------------------------------------------
  data transmit mode operation
--------------------------------------------------*/

                if ( gp_tx_set != gp_tx_get )
                {

/*----------------------------------------------
    more data to transmit
----------------------------------------------*/

                    g_status = TX_MODE;         /* set mode to transmit mode    */
                    IICA0 = g_tx_buff[gp_tx_get]; /* set transmit data          */
                    ++gp_tx_get;                /* move pointer to next data    */

                }
                else
                {
/*----------------------------------------------
    no more data to transmit
----------------------------------------------*/

                    g_status = TX_END;          /* set data transfer end        */
                    g_erflag = 0x00;            /* set success flag             */
                    r_operation_end();          /* issue restart or stop        */

                }

            }   /* end of data transmit */
            else
            {

/*--------------------------------------------------
    data receive mode operation
--------------------------------------------------*/

                WTIM0 = 0;                      /* set 8bit wait mode           */
                ACKE0 = 1;
                
                IICA0 = DUMMY_DATA;             /* canceling wait and start Rx  */
                g_status = RX_MODE;             /* set receive mode flag        */

            }   /* end of data receive start    */

        }   /* end of ACK for slave address */
        else
        {

/*----------------------------------------------
    NAK response for slave address
----------------------------------------------*/

            g_status = NO_SLAVE;                /* set communication error flag */
            g_erflag = 0x02;                    /* set error flag(No slave)     */
            r_operation_end();                  /* issue restart or stop        */

        }   /* end of no slave exit */

    }   /* end of slave address transmit operation  */
    else
    {

/*======================================================
    data transfer stage(STD0 is 0)
======================================================*/

        if ( 1 == TRC0 )                        /* check transmit or receive    */
        {

/*--------------------------------------------------
    transmit operation mode
--------------------------------------------------*/

            if ( 1 == ACKD0 )                   /* check slave state            */
            {                                   /* slave is ready for next data */

/*----------------------------------------------
  success data transfer and check remaining data
----------------------------------------------*/

                if ( gp_tx_set != gp_tx_get )
                {

/*----------------------------------------------
    more data to transmit
----------------------------------------------*/

                    IICA0 = g_tx_buff[gp_tx_get]; /* set transmit data          */
                    ++gp_tx_get;                /* move pointer to next data    */

                }   /* end of a data transmit   */
                else
                {
/*----------------------------------------------
    no more data to transmit
----------------------------------------------*/

                    g_status = TX_END;          /* set data transfer end        */
                    g_erflag = 0x00;            /* clear error flag             */
                    r_operation_end();          /* operation end                */

                }   /* end of data transmission */

            }   /* end by ACK response  */
            else
            {
/*----------------------------------------------
    NACK error
----------------------------------------------*/

                g_status = NO_ACK;              /* set communication error flag */
                g_erflag = 0x03;                /* set error(No ACK for data)   */
                r_operation_end();              /* operation end                */

            }   /* end by NACK response */

        }   /* end of data transmit */
        else
        {

/*--------------------------------------------------
    receive operation mode
--------------------------------------------------*/

            if ( 0 != g_rx_num )                /* check received data number   */
            {

/*----------------------------------------------
store recieved data
----------------------------------------------*/

                g_rx_buff[gp_rx_set] = IICA0;   /* store received data          */
                ++gp_rx_set;                    /* move store pointer           */
                --g_rx_num;                     /* count down data number       */
                if ( 0 == g_rx_num )
                {

/*----------------------------------------------
receive last data operation (send NACK and change to 9 clock wait)
----------------------------------------------*/

                    ACKE0 = 0;                  /* NACK for last data           */
                    WTIM0 = 1;                  /* change to 9clock wait        */
                    WREL0 = 1;                  /* canceling wait to end        */
                }
                else
                {
                    WREL0 = 1;                  /* start next reception         */
                }

            }   /* end of receive continue  */
            else
            {

/*----------------------------------------------
receive operation complete at 9th clock
----------------------------------------------*/

                g_status = RX_END;              /* set data receive end         */
                g_erflag = 0x00;                /* clear error flag             */
                r_operation_end();              /* operation end                */

            }   /* end of data number check of receive  */

        }   /* end of data receive  */

    }   /* end of data transfer */

/* master transfer operation end    */

}  /* end of function r_IICA0_interrupt */


/***********************************************************************
* Function Name: r_operation_end
* Description  : This function end of  service routine.
* Arguments    : none
* Return Value : ressult
*                
* Remark       : 
************************************************************************/
void r_operation_end(void)
{

    uint8_t status;

    if ( 0 != g_stop_flag )                     /* check stop flag              */
    {                                           /* issue stop condition         */

/*----------------------------------------------
    release I2C bus( issue stop condition)
----------------------------------------------*/

        status = r_stop_condition();            /* issue stop condition         */
        if ( 00 != status )
        {                                       /* if fail re-start             */

            if ( 00 == g_status )               /* check previous status        */
            {                                   /* if no error then change      */

                g_status = BUS_ERROR;           /* set bus error flag           */
                g_erflag = 0x04;                /* set error flag               */

            }

        }
        else
        {

            if ( 00 == g_status )               /* check previous status        */
            {                                   /* if no error then change      */

                g_status = BUS_FREE;            /* set BUS FREE status          */
                g_erflag = SUCCESS;             /* set success flag             */

            }

        }
    }

}   /* end of r_operation_end   */


/***********************************************************************
* Function Name: r_start_condition
* Description  : This function issue start condition
* Arguments    : none
* Return Value : bus status
*               SUCCESS(0x00)   : success to get I2C bus
*               BUS_ERROR(0x8F) : fail to get I2C bus
* Remark       : 
************************************************************************/
uint8_t r_start_condition(void)
{

    uint8_t status;
    uint16_t count;

    STT0 = 1;                                   /* trigger start condition      */

    count = LOOPLIMIT;                          /* set time limit               */

    while ( 0 == STD0 )                         /* wait for start condition     */
    {
        --count;                                /* count down limit time        */
        if ( 0 == count )
        {
            break;                              /*exit if time over             */
        }
    }

    if ( ( 1 == MSTS0 ) && ( 1 == STD0) )
    {
        status = SUCCESS;                       /* success to get I2C bus       */
    }
    else
    {
        status = BUS_ERROR;                     /* fail to get I2C bus          */
    }

    return(status);

}  /* end of function r_start_condition */


/***********************************************************************
* Function Name: r_stop_condition
* Description  : This function issue stop condition
* Arguments    : none
* Return Value : bus status
*               SUCCESS(0x00)   : success to release I2C bus
*               BUS_ERROR(0x8F) : fail to release I2C bus
* Remark       : 
************************************************************************/
uint8_t r_stop_condition(void)
{

    uint8_t status;
    uint16_t count;

    SPT0        = 1;                            /* trigger stop condition       */

    count = LOOPLIMIT;                          /* set time limit               */

    while ( 0 == SPD0 )                         /* wait for stopcondition       */
    {
        --count;                                /* count down limit time        */
        if ( 0 == count )
        {
            break;                              /* exit if time over            */
        }
    }

    if ( 1 == SPD0 )
    {
        status = SUCCESS;                       /* success to release I2C bus   */
    }
    else
    {
        status = BUS_ERROR;                     /* fail to release I2C bus      */
    }

    return(status);

}  /* end of function r_stop_condition */


