/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/*******************************************************************************
* File Name    : AR_LIB_WIRE1.c
* Version      : Rev. 1.00
* Device(s)    : R5F104ML
* Tool-Chain   : CCRL
* Description  : This file implements Arduino library(Wire1).
* Creation Date: 2021/05/14
********************************************************************************/

/*******************************************************************************
Includes
********************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "r_cg_timer.h"
#include "r_cg_serial.h"
#include <AR_LIB_TIME.h>

#ifdef USE_WIRE1

#include "AR_LIB_WIRE1.h"
#include <stdarg.h>
#include <stdio.h>
#include "AR_WIRE_DEF.h"

/*******************************************************************************
Pragma directive
********************************************************************************/
#ifdef   CS_ENV
#pragma interrupt r_IIC00_interrupt(vect=INTIIC00)
#pragma interrupt r_TM10_interrupt(vect=INTTM10,bank=RB3,enable=true)
#endif


/*******************************************************************************
Global variables and functions
********************************************************************************/

/*--------------------------------------------------------------
    IIC00(wire1)
---------------------------------------------------------------*/

volatile    uint8_t gp_tx1_set;          /* transmit data set pointer            */
volatile    uint8_t gp_tx1_get;          /* transmit data read pointer           */
volatile    uint8_t g_tx1_buff[256];     /* I2C transmit data queue              */

volatile    uint8_t gp_rx1_set;          /* received data set pointer            */
volatile    uint8_t gp_rx1_get;          /* received data read pointer           */
volatile    uint8_t g_rx1_buff[256];     /* I2C recived data queue               */

volatile    uint16_t g_rx1_num;          /* remaining receive data number        */
volatile    uint8_t sladdr8_1;           /* 8-bit slave address                  */

volatile    uint8_t g_stop_flag_1;       /* stop issue flag 0:do not/1:issue     */

volatile    uint8_t g_status_1;          /* IIC00 status
    0x00 : BUS_FREE     :BUS FREE
    0x8F : BUS_ERROR    :BUS Error
    0x10 : GET_BUS      :get I2C bus
    0x20 : GET_BUS4TX   :get I2C bus to transmit
    0x30 : TX_MODE      :Transmit operation
    0x40 : TX_END       :Transmit end
    0x50 : GET_BUS4RX   :get bus to receive
    0x60 : RX_MODE      :Receive operation
    0x70 : RX_END       :Receive end
    0x81 : BUFF_OVER    :Buffer over
    0x82 : NO_SLAVE     :No slave
    0x83 : NO_ACK       :No ACK for data
*/

volatile    uint8_t g_erflag_1;         /* wire function result status
    0x00 :  success
    0x01 :  buffer over flow
    0x02 :  No ACK for slave address
    0x03 :  No ACK for data
    0x04 :  other error
    */


/*==============================================================

    definition of wire API functions

===============================================================*/

/*--------------------------------------------------------------
    IIC00(wire1)
---------------------------------------------------------------*/
API_Wire Wire1 = {
    Wire1_begin,                        /*start of wire function as master      */
    Wire1_beginTransmission,            /* start transmit to slave              */
    Wire1_writeb,                       /* data set to transmit buffer          */
    Wire1_writec,                       /* data set to transmit buffer          */
    Wire1_endTransmission,              /* end transmit to slave                */
    Wire1_requestFrom,                  /* start receiving                      */
    Wire1_requestFromS,                 /* start receiving                      */
    Wire1_available,                    /* check receive buffer                 */
    Wire1_read,                         /* read received data from buffer       */
};


/* start receive data from slave    */
void Wire1_requestFromSub(uint8_t saddr7, uint16_t bytes);


void r_operation_end1(void);            /* end to transfer, keep or lerease bus */
uint8_t r_start_condition1(void);       /* isuue start condition                */
uint8_t r_stop_condition1(void);        /* isuue stop condition                 */
void r_iic00_stop(void);                /* stop IIC00 operation                 */


/***********************************************************************
* Function Name: Wire1_begin
* Description  : This function initialize Wire1 to use as master
* Arguments    : none
* Return Value : none
* Remark       : 
************************************************************************/
void Wire1_begin(void)
{

    gp_tx1_set = 0;                      /* initialize pointers for data queues  */
    gp_tx1_get = 0;
    gp_rx1_set = 0;
    gp_rx1_get = 0;
    
    R_IIC00_Create();                   /* re_initialize IIC00                  */

}   /* end of function Wire1_begin */


/***********************************************************************
* Function Name: Wire1_beginTransmission
* Description  : This function start write to slave
* Arguments    : saddr7: slave address of 7 bit
* Return Value : g_erflag_1
*               = 00 : success
*                 04 : fail
************************************************************************/
uint8_t Wire1_beginTransmission(uint8_t saddr7)
{

    g_erflag_1 = 0x04;                  /* set dummy error flag                 */

    if ( RX_MODE == g_status_1 )        /* if receive mode exit with error flag */
    {

        return(g_erflag_1);

    }

    sladdr8_1   = (saddr7 << 1);        /* get 8bit slave address               */

    g_status_1 = r_start_condition1();  /* issue start condition                */

    if ( 0 == g_status_1 )
    {

        gp_tx1_set = 0;                 /* initialize pointers for data queues  */
        gp_tx1_get = 0;

        g_status_1 = GET_BUS4TX;        /* set status to "getting I2C bus"      */
        g_erflag_1 = SUCCESS;           /* set success flag                     */

    }

    return(g_erflag_1);

}   /* end of function Wire1_beginTransmission  */


/***********************************************************************
* Function Name: Wire1_writec
* Description  : This function set a charactor to transmit buffer for Wire1
* Arguments    : data
* Return Value : data number in transmit data queues
* Remark       : g_status_1 = GET_BUS4TX(0x20) : success
*                             BUS_ERROR(0x8F)  : fail
*                             BUFF_OVER(0x81)  : fail
*               g_erflag_1  = 00
************************************************************************/
uint8_t Wire1_writec(uint8_t data)
{

    if ( 250 <= gp_tx1_set )
    {

        g_status_1 = BUFF_OVER;         /* set data over flag                   */
        g_erflag_1 = 0x01;              /* set buffer over flow flag            */
        return(gp_tx1_set);

    }   /* end of data over error   */

    g_erflag_1 = 0x04;                  /* set dummy error flag                 */

    if ( GET_BUS4TX == g_status_1 )
    {

        g_tx1_buff[gp_tx1_set] = data;  /* set transmit data in data queues     */
        ++gp_tx1_set;                   /* move pointer to next address         */
        g_erflag_1 = SUCCESS;           /* set success flag                     */

    }

    return(gp_tx1_set);                 /* return data number in queues         */

}   /* end of function Wire1_writec */


/***********************************************************************
* Function Name: Wire1_writeb
* Description  : This function set a block data to transmit buffer
* Arguments    : data
* Return Value : data number in transmit data queues
* Remark       : g_status_1 = GET_BUS4TX(0x20) : success
*                             BUS_ERROR(0x8F)  : fail
*                             BUFF_OVER(0x81)  : fail
************************************************************************/
uint8_t Wire1_writeb(uint8_t *buff, uint8_t bytes)
{

    uint8_t loop;
    uint8_t *ptr;

    if ( 250 <= ( gp_tx1_set + bytes ))
    {

        g_status_1 = BUFF_OVER;         /* set data over flag                   */
        g_erflag_1 = 0x01;              /* set buffer over flow flag            */
        return(gp_tx1_set);

    }   /* end of data over error   */

    g_erflag_1 = 0x04;                  /* set dummy error flag                 */

    if ( GET_BUS4TX == g_status_1 )
    {

        ptr = buff;

        for ( loop = 0 ; loop < bytes ; loop++ )
        {

            g_tx1_buff[gp_tx1_set] = *ptr; /* set transmit data in data queues  */
            ++gp_tx1_set;               /* move set pointer to next address     */
            ++ptr;                      /* move read pointer                    */

        }   /* end of data set  */

        g_erflag_1 = SUCCESS;           /* set success flag                     */

    }

    return(gp_tx1_set);                 /* return data number in queues         */


}   /* end of function Wire1_writeb  */


/***********************************************************************
* Function Name: Wire1_endTransmission
* Description  : This function actually start write data to slave
* Arguments    : stop  : FALSE(0):do not issue STOP /TRUTH(1):issue STOP
* Return Value : g_erflag_1 = 0x00 : success
*                             0x02 : NACK for slave address
*                             0x03 : NACK for transmit data
*                             0x04 : other error
* Remark       : g_status_1 = 0x10 : success
*                             0x8F : fail
************************************************************************/
uint8_t Wire1_endTransmission(uint8_t stop)
{

    if ( GET_BUS4TX != g_status_1 )     /* check if transmit mode or not        */
    {

        g_status_1 = BUS_ERROR;         /* set bus error flag                   */
        g_erflag_1 = 0x04;
        return(g_erflag_1);             /* exit with error flag                 */

    }   /* end of I2C bus error procedure   */

    g_stop_flag_1 = stop;               /* set issue stop condition flag        */

    IICIF00 = 0;                        /* clear INTIIC00 request flag          */
    IICMK00 = 0;                        /* enable INTIIC00                      */
    TDR10 = TX_DELAY1;                  /* set delay time for transmit interval */
    SIO00 = sladdr8_1;                  /* transmit slave address               */


/*--------------------------------------------------
  transmission start and  wait for transmission end
--------------------------------------------------*/

    while ( TX_MODE == ( g_status_1 | 0x10 ) )  /* wait for transmission end    */
    {
        NOP();
    }

    return(g_erflag_1);                         /* exit with result flag        */
    
}   /* end of function Wire1_endTransmission  */


/***********************************************************************
* Function Name: Wire1_requestFromS
* Description  : This function start read from slave with STOP condition
* Arguments    : 
*               saddr7: slave address of 7 bit
*               bytes : receive data number
* Return Value : g_erflag_1 = 0x00 : success
*                             0x01 : buffer over flow
* Remark       : g_status_1 = 0x40 : success
*                             0x81 : data number error
*                             0x8F : fail
************************************************************************/
uint8_t Wire1_requestFromS(uint8_t saddr7, uint16_t bytes)
{

    g_stop_flag_1 = TRUTH;              /* set issue stop flag                  */

    Wire1_requestFromSub(saddr7, bytes);

    return(g_erflag_1);

}   /* end of function Wire1_requestFromS   */


/***********************************************************************
* Function Name: Wire1_requestFrom
* Description  : This function start read datas from slave with STOP condition
* Arguments    : 
*               saddr7: slave address of 7 bit
*               bytes : receive data number
*               stop  : FALSE(0):do not issue STOP /TRUTH(1):issue STOP
* Return Value : g_erflag_1 = 0x00 : success
*                             0x01 : buffer over flow
* Remark       : g_status_1 = 0x40 : success
*                             0x81 : data number error
*                             0x8F : fail
************************************************************************/
uint8_t Wire1_requestFrom(uint8_t saddr7, uint16_t bytes, uint8_t stop)
{

    g_stop_flag_1 = stop;               /* set issue stop condition flag        */

    Wire1_requestFromSub(saddr7, bytes);

    return(g_erflag_1);

}  /* end of function Wire1_requestFrom */


/***********************************************************************
* Function Name: Wire1_requestFromSub
* Description  : This function issue start condition for read
* Arguments    : 
*               saddr7: slave address of 7 bit
*               bytes : receive data number
* Return Value : none
* Remark       : g_status_1 = 0x50 : success
*                             0x81 : data number error
*                             0x84 : no data is requested
*                             0x8F : fail
*                g_erflag_1 = 0x00 : success
*                             0x01 : buffer over flow
*                             0x04 : other error
************************************************************************/
void Wire1_requestFromSub(uint8_t saddr7, uint16_t bytes)
{

    g_erflag_1 = 0x00;                  /* clear error flag                     */

    if ( 255 <= bytes )                 /* check data number to receive         */
    {

        g_status_1 = BUFF_OVER;         /* requested too many number of data    */
        g_erflag_1 = 0x01;              /* set error status set                 */
        return;

    }   /* end of data over error   */

    if ( 0 == bytes )
    {

        g_status_1 = NO_DATA;           /* no data is requested                 */
        g_erflag_1 = 0x04;              /* set no data is requested error flag  */
        return;

    }   /* end of data number error */

/*--------------------------------------------------
  check I2C bus usage
--------------------------------------------------*/

    if (( GET_BUS4TX == g_status_1 ) 
        || ( RX_MODE == g_status_1) 
        || ( GET_BUS4RX == g_status_1)
        )
    {

        g_erflag_1 = 0x04;              /* set error flag                       */
        return;                         /* exit with error flag                 */

    }   /* end of I2C bus error */

/*--------------------------------------------------
  end of parameter check  and start operation
--------------------------------------------------*/

    g_rx1_num    = (uint8_t)bytes;      /* set recive data number               */
    gp_rx1_set   = 0x00;                /* initialize buffer pointer            */
    gp_rx1_get   = 0x00;

    sladdr8_1    = (saddr7 << 1);       /* get 8bit slave address               */

    g_status_1 = r_start_condition1();  /* issue start condition                */

    if ( 0 == g_status_1 )
    {                                   /* success to issue start condition     */

        IICIF00 = 0;                    /* clear INTIIC00 request flag          */
        IICMK00 = 0;                    /* enable INTIIC00                      */

        g_status_1 = GET_BUS4RX;        /* set slave address transmit mode      */
        TDR10 = RX_DELAY1;              /* set delay time for read intervl      */
        SIO00 = ( sladdr8_1 | 0x01 );   /* transmit slave address for receive   */

    }

    /*
    transfer is done as back ground operation.
    g_status_1 showes status of operation.
    g_erflag_1 showes the error
    and Wire1_availabe() can get received data number.
    */


}  /* end of function Wire1_requestFromSub */


/***********************************************************************
* Function Name: Wire1_available
* Description  : This function return available data number in receive buffer
* Arguments    : none
* Return Value : data number in receive buffer
* Remark       : 
************************************************************************/
uint8_t Wire1_available(void)
{

    uint8_t work;

    work = gp_rx1_set - gp_rx1_get;     /* get remaining data number            */ 

    return( work );                     /* return remaining data number         */

}   /* end of function Wire1_available  */



/***********************************************************************
* Function Name: Wire1_read
* Description  : This function return data at top of buffer
* Arguments    : none
* Return Value : data or 0x00(if there is no more data)
* Remark       : 
************************************************************************/
uint8_t Wire1_read(void)
{

    uint8_t work;

    work = gp_rx1_set - gp_rx1_get;     /* get remaining data number            */ 

    if ( 0x00 != work )
    {

        work = g_rx1_buff[gp_rx1_get];  /* get data from buffer                 */
        ++gp_rx1_get;                   /* move read pointer                    */

    }
    else
    {

        g_erflag_1 = 0x01;              /* set error flag                       */

    }

    return( work );

}   /* end of function Wire1_read   */


/***********************************************************************
* Function Name: r_IIC00_interrupt
* Description  : This function only trigger TM10(actual service routine).
* Arguments    : none
* Return Value : none
* Remark       : 
************************************************************************/
#ifdef  CS_ENV
static void __near r_IIC00_interrupt(void)
#endif
#ifdef  IAR_ENV
#pragma vector = INTIIC00_vect
__interrupt void r_IIC00_interrupt(void)
#endif
{

    TS1 = TM10_TRG;                     /* start one shot timer for delay time  */

}  /* end of function r_IIC00_interrupt */


/***********************************************************************
* Function Name: r_TM10_interrupt
* Description  : This function performs actual INTIIC00.
* Arguments    : none
* Return Value : none
* Remark       : 
************************************************************************/
#ifdef  CS_ENV
static void __near r_TM10_interrupt(void)
{
#endif
#ifdef  IAR_ENV
#pragma vector = INTTM10_vect
__interrupt void r_TM10_interrupt(void)
{
    EI();
#endif

    uint8_t state;

    state = ( SSR00L & 0x02 );          /* get NACK error flag                  */
    SIR00L = state;                     /* clear error flag if error            */

/*==================================================================
  check communication status(g_status_1) and proceed each function
    0x20 : GET_BUS4TX   :have I2C bus and start to transmit
    0x30 : TX_MODE      :Transmit operation
    0x50 : GET_BUS4RX   :keep I2C bus and start to receive
    0x60 : RX_MODE      :Receive operation
==================================================================*/

    switch ( g_status_1 )
    {

/*==================================================================
  transmit function
    0x20 : GET_BUS4TX   :have I2C bus and start to transmit
    0x30 : TX_MODE      :Transmit operation
==================================================================*/

        case GET_BUS4TX:

/*======================================================
    g_status_1 is 0x20 : slave address transmission end
======================================================*/

            if ( 0x00 == state)                     /* check ACK response       */
            {

/*----------------------------------------------
    ACK response for slave address
----------------------------------------------*/

                if ( gp_tx1_set == gp_tx1_get )     /* check data number        */
                {

/*----------------------------------------------
    there is no data to transmit
----------------------------------------------*/

                    g_status_1 = TX_END;            /* set data transfer end    */
                    g_erflag_1 = SUCCESS;           /* set success flag         */
                    r_operation_end1();             /* issue restart or stop    */

                }
                else
                {

/*----------------------------------------------
    there are more data to transmit
----------------------------------------------*/

                    g_status_1 = TX_MODE;           /* change mode to transmit  */
                    SIO00 = g_tx1_buff[gp_tx1_get]; /* transmit next data       */
                    ++gp_tx1_get;                   /* move pointer to next data*/

                }

            }
            else
            {

/*----------------------------------------------
    no ACK for slave address
----------------------------------------------*/

                g_status_1 = NO_SLAVE;      /* set error to status flag         */
                g_erflag_1 = 0x02;          /* set no ACK for slave address flag*/
                r_operation_end1();         /* issue restart or stop            */

            }

            break;


        case TX_MODE:

/*======================================================
    g_status_1 is 0x30 : data transmit end
======================================================*/

            if ( 0x00 == state)                     /* check ACK for data       */
            {

                if ( gp_tx1_set == gp_tx1_get )     /* check data number        */
                {

/*----------------------------------------------
    there is no data to transmit
----------------------------------------------*/

                    g_status_1 = TX_END;            /* set data transfer end    */
                    g_erflag_1 = SUCCESS;           /* set success flag         */
                    r_operation_end1();             /* operation end            */

                }
                else
                {

/*----------------------------------------------
    ther are more data to transmit
----------------------------------------------*/

                    SIO00 = g_tx1_buff[gp_tx1_get]; /* transmit next data       */
                    ++gp_tx1_get;                   /* move pointer to next data*/

                }

            }
            else
            {

/*----------------------------------------------
    no ACK for transmit data
----------------------------------------------*/

                g_status_1 = NO_ACK;        /* set error to status flag         */
                g_erflag_1 = 0x03;          /* set no ACK for transmit data flag*/
                r_operation_end1();         /* operation end                    */

            }

            break;

/*==================================================================
  Wire1 receive function
    0x50 : GET_BUS4RX   :keep I2C bus and start to receive
    0x60 : RX_MODE      :Receive operation
==================================================================*/

        case GET_BUS4RX:

/*======================================================
    g_status_1 is 0x50 : slave address transmission end
======================================================*/

            if ( 0x00 == state)         /* check ACK response for slave address */
            {

/*----------------------------------------------
    detect ACK response from slave
----------------------------------------------*/

                SCR00 &= (~0x0C000);    /* disable transmit                     */
                SCR00 |= 0x4000;        /* enable receive operation             */

                if ( 1 == g_rx1_num )   /* check remaining data number          */
                {

/*-------------------------------------------------------------------
    next data is last data to receive. then prepare NACK response
---------------------------------------------------------------------*/

                    SOE0 &= (~IIC00_TRG);   /* to prevent ACK response          */

                }

/*----------------------------------------------
    start to receive next data
----------------------------------------------*/

                SIO00 = DUMMY_DATA;         /* trigger communication            */
                g_status_1 = RX_MODE;       /* change transfer mode flag        */

            }
            else
            {

/*----------------------------------------------
    no ACK for slave address
----------------------------------------------*/

                g_status_1 = NO_SLAVE;      /* set error to status flag         */
                g_erflag_1 = 0x02;          /* set no ACK for slave address flag*/
                r_operation_end1();         /* operation end                    */

            }

            break;


        case RX_MODE:

/*======================================================
    data transfer end for receive operation(0x60)
======================================================*/

/*----------------------------------------------
    store received data
----------------------------------------------*/

            g_rx1_buff[gp_rx1_set] = SIO00; /* store received data to buffer    */
            ++gp_rx1_set;                   /* move data store pointer          */
            --g_rx1_num;                    /* count down data number           */

/*----------------------------------------------
    check remaining data number
----------------------------------------------*/

            switch ( g_rx1_num )
            {


                case 0x00:

/*----------------------------------------------
    all data is received yet 
----------------------------------------------*/

                    SOE0 |= IIC00_TRG;      /* end of NACK response             */
                    g_status_1 = RX_END;    /* set receive end flag             */
                    g_erflag_1 = SUCCESS;   /* set operation success flag       */
                    r_operation_end1();     /* operation end                    */

                    break;


                case 0x01:

/*----------------------------------------------
    next data is last data
----------------------------------------------*/

                    SOE0 &= (~IIC00_TRG);   /* to prevent ACK response          */


                default:

/*----------------------------------------------
    more data to receive and start to receive
----------------------------------------------*/

                    SIO00 = DUMMY_DATA;     /* trigger next communication       */
                    break;

            }   /* end of switch ( g_rx1_num )  */
                
        default:

            break;

    }   /* end of switch ( g_status_1 ) */

}  /* end of function r_TM10_interrupt */


/***********************************************************************
* Function Name: r_operation_end1
* Description  : This function end of  service routine.
* Arguments    : none
* Return Value : ressult
*                
* Remark       : if g_stop_flag_1 is 0 then restart(issue start condition)
*              : if g_stop_flag_1 is not 0 then stop(issue stop condition)
************************************************************************/
void r_operation_end1(void)
{

    uint8_t status;

    if ( 0 != g_stop_flag_1 )                   /* check stop flag              */
    {                                           /* issue stop condition         */

/*----------------------------------------------
    release I2C bus( issue stop condition)
----------------------------------------------*/

        status = r_stop_condition1();           /* issue stop condition         */
        if ( 00 != status )
        {                                       /* if fail re-start             */

            if ( 00 == g_status_1 )             /* check previous status        */
            {                                   /* if no error then change      */

                g_status_1 = BUS_ERROR;         /* set bus error flag           */
                g_erflag_1 = 0x04;              /* set error flag               */

            }

        }
        else
        {

            if ( 00 == g_status_1 )             /* check previous status        */
            {                                   /* if no error then change      */

                g_status_1 = BUS_FREE;          /* set BUS FREE status          */
                g_erflag_1 = SUCCESS;           /* set success flag             */

            }

        }

    }   /* end of if(0 != g_stop_flag_1)    */

    IICIF00 = 0;;                               /* clear INTIIC00 request flag  */
    IICMK00 = 1;                                /* disable INTIIC00             */

}   /* end of r_operation_end1  */


/***********************************************************************
* Function Name: r_start_condition1
* Description  : This function issue start condition
* Arguments    : none
* Return Value : bus status
*               SUCCESS(0x00)   : success to get I2C bus
*               BUS_ERROR(0x8F) : fail to get i2C bus
* Remark       : 
************************************************************************/
uint8_t r_start_condition1(void)
{

    uint8_t state = SUCCESS;

    r_iic00_stop();                             /* stop IIC00 operation         */

/*------------------------------------------------
    prepare for start condition
------------------------------------------------*/

    delayMicroseconds(5);                       /* wait for bus free time       */

    if ( FALSE  == (SO0 & IIC00_TRG) )          /* check SDA signal level       */
    {                                           /* if low level then            */

        SO0 |= IIC00_TRG;                       /* rise SO00 bit(SDA)           */
        delayMicroseconds(5);                   /* wait for bus free time       */

    }

    if ( FALSE == (SO0 & IIC00_CK) )            /* check SCL signal level       */
    {                                           /* if SCL level is low then     */

        SO0 |= IIC00_CK;                        /* rise up SCL signal           */
        delayMicroseconds(5);

    }

    PMS = 0x01;                                 /* set read external pin mode   */

    if ( ( 0 == P3_bit.no0 ) || ( 0 ==P5_bit.no0 )) /* check SCL & SDA are High */
    {

        PMS = 0x00;                             /* set normal read mode         */
        return(BUS_ERROR);                      /* both are not high then error */

    }

    PMS = 0x00;                                 /* set normal read mode         */

/*------------------------------------------------
    issue start condition
------------------------------------------------*/

    SO0 &= (~IIC00_TRG);                        /* fall down SDA when in SCL=H  */

    delayMicroseconds(5);                       /* wait for bus setup time      */

    SO0 &= (~IIC00_CK);                         /* fall down SCL after SDA =L   */

/*------------------------------------------------
    enable IIC00 operation
------------------------------------------------*/

    SOE0 |= IIC00_TRG;                          /* enable channel00 output      */
    SCR00 &= (~0xC000);                         /* clear TX and RX enable bit    */
    SCR00 |= 0x8000;                            /* set transmit enable          */
    SS0 = IIC00_TRG;                            /* enable channel00 operation   */

    return(state);

}  /* end of function r_start_condition1 */


/***********************************************************************
* Function Name: r_stop_condition1
* Description  : This function issue stop condition
* Arguments    : none
* Return Value : bus status
*               SUCCESS(0x00)   : success to release I2C bus
*               BUS_ERROR(0x8F) : fail to get i2C bus
* Remark       : 
************************************************************************/
uint8_t r_stop_condition1(void)
{

    uint8_t state = SUCCESS;

    delayMicroseconds(5);                       /* 5us for bus free time        */

    r_iic00_stop();                             /* stop IIC00 operation         */

/*------------------------------------------------
    prepare for stop condition
------------------------------------------------*/

    SO0 &= (~IIC00_TRG);                        /* fall down SDA signal         */

    delayMicroseconds(5);                       /* for command interval         */

    SO0 |= IIC00_CK;                            /* rise up SCL signal           */

    delayMicroseconds(5);                       /* for 5us for setup time       */

/*------------------------------------------------
    issue stop condition
------------------------------------------------*/

    SO0 |= IIC00_TRG;                           /* rise up SDA signal           */

    delayMicroseconds(10);                      /* 10us for bus free time       */

    SCR00 &= (~0xC000);                         /* clear TX and RX enable bit    */

/*------------------------------------------------
    check SDA signal level
------------------------------------------------*/

    PMS = 0x01;                                 /* set read external pin mode   */
    if ( 0 == P5_bit.no0 )                      /* check SDA signal level       */
    {

        state = BUS_ERROR;                      /* if SDA is low then set error */

    }

    PMS = 0x00;                                 /* set normal read mode         */

    return(state);

}  /* end of function r_stop_condition1 */


/***********************************************************************
* Function Name: r_iic00_stop
* Description  : This function stop IIC00 operation
* Arguments    : none
* Return Value : none
************************************************************************/
void r_iic00_stop(void)
{

    uint8_t work;

    delayMicroseconds(5);                       /* dummy wait 5us               */

    work = 0x40 & SSR00L;                       /* check communication status   */

    if ( FALSE != work )
    {

/*------------------------------------------------
    communication continues
------------------------------------------------*/

        while ( FALSE != work )                 /* wait for communication end   */
        {

            work = SSR00L;                      /* get IIC00 status             */
            work &= 0x40;                       /* get transfer status (TSF)    */

        }

        delayMicroseconds(5);                   /* wait for hold time(5us)      */

    }

/*------------------------------------------------
    communication end then stop IIC00 operation
------------------------------------------------*/

    SOE0 &= (~IIC00_TRG);                       /* disable IIC00 output         */
    ST0L = IIC00_TRG;                           /* stop channel 0 operation     */

}  /* end of function r_iic00_stop  */

#endif


