/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/*******************************************************************************
* File Name    : AR_LIB_ANALOG.c
* Version      : Rev. 1.00
* Device(s)    : R5F104ML
* Tool-Chain   : CCRL
* Description  : This file implements Arduino library(analog I/O).
* Creation Date: 2020/07/01
********************************************************************************/

/*******************************************************************************
Includes
********************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "r_cg_timer.h"
#include "AR_LIB_ANALOG.h"
#include "AR_LIB_TIME.h"
/*******************************************************************************
Pragma directive
********************************************************************************/
#ifdef   CS_ENV
#pragma interrupt r_tau0_channel0_interrupt(vect=INTTM00,bank=RB1)
#endif


/*******************************************************************************
Global variables and functions
********************************************************************************/

#define D3PORT  ( P7 )                          /* D3 pin is connected P7       */
#define D5PORT  ( P7 )                          /* D5 pin is connected P7       */
#define D11PORT ( P5 )                          /* D11 pin is connected P5      */

#define D3_ON   ( 0x20 )                        /* D3 on data                   */
#define D3_OFF  ( 0xDF )                        /* D3 off data                  */
#define D5_ON   ( 0x80 )                        /* D5 on data                   */
#define D5_OFF  ( 0x7F )                        /* D5 off data                  */
#define D11_ON  ( 0x04 )                        /* D11 on data                  */
#define D11_OFF ( 0xFB )                        /* D11 off data                 */

/*******************************************************************************
port definition for G14_FPB
********************************************************************************/


volatile    uint8_t g_dachannel[6] =
{
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00
};  /* D/A convert usage flag       */
/*  request data area
 order channel
    0   D3 (P75) 0 : 0%(not used)/1-255 : used
    1   D5 (P77) 0 : 0%(not used)/1-255 : used
    2   D11(P52) 0 : 0%(not used)/1-255 : used
    3   D6 (P10) 0 : 0%(not used)/1-255 : used
    4   D9 (P11) 0 : 0%(not used)/1-255 : used
    5   D10(P12) 0 : 0%(not used)/1-255 : used
*/

/* software PWM control data area   */
volatile __saddr uint8_t g_da_time[3];          /* 0-255: D/A data (work area)  */
volatile __saddr uint8_t g_times = 0xFE;        /* interrupt counter            */


uint16_t conv_TRD1(uint8_t value);              /* convert PWM high time to TDR */

/*******************************************************************************
* Function Name: anlogRead
* Description  : This function inputs analog signal from specfied pin.
* Arguments    : pin:pin number( 0(A0) - 5(A5) )
* Return Value : result of A/D conversion
********************************************************************************/
uint16_t analogRead(uint8_t pin)
{

    uint16_t work = 0x0000;                     /* clear data area          */
    uint8_t loop;

/*=======================================
    check analog signal pin
=======================================*/

    if( pin > 5 ) return( 0xFFFF );             /* exit if pin number over 6    */

/*=======================================
    get pin value
=======================================*/

    ADS = ( 6 - pin );                          /* set analog input pin         */
    ADM0 |= 0x01;                               /* turn A/D comparator power on */
    delayMicroseconds(1);                       /* wait for 1 microsecond       */

    for ( loop = 0 ; loop < 4 ; loop++)
    {
        ADM0 |= 0x80;                           /* trigger A/D conversion       */
        ADIF = 0;                               /* clear A/D interrupt flag     */

        while ( 0 == ADIF )
        {
            NOP();
        }

        ADIF = 0;

        work += ( ADCR >> 6 );

    }

    ADM0 &= 0x7E;                               /* stop A/D conversion          */

    work >>= 2;                                 /* get average of 4times        */

    return( work );                             /* return with A/D result       */

}   /* end of analogRead function   */


/*******************************************************************************
* Function Name: analogWrite
* Description  : This function set pin to PWM.
* Arguments    : pin:pin number( D3/D5/D6/D9/D10/D11 )
                 value:analog data 0-0xFF
* Return Value : error flag
*               0x00 : success
********************************************************************************/
uint8_t analogWrite(uint8_t pin,uint8_t value)
{

    switch ( pin )
    {

        case 3:                                 /* use D3 pin as PWM signal */

                /* set new value to buffer  */
            g_dachannel[0] = value;             /* turn operation mode ON       */

            PM7 &= D3_OFF;                      /* set port to output           */


            break;


        case 5:                                 /* use D5 pin as PWM signal     */

                /* set new value to buffer  */
            g_dachannel[1] = value;             /* turn operation mode ON       */

            PM7 &= D5_OFF;                      /* set port to output           */

            break;


        case 6:                                 /* use D6 pin as PWM signal     */

            g_dachannel[3] = value;             /* turn operation mode ON       */
            TRDGRD1 = conv_TRD1( value );       /* set high time to TRD1        */
            PM1 &= 0xFE;                        /* set port to output           */
            P1 &= 0xFE;                         /* set port to low              */

            break;


        case 9:                                 /* use D9 pin as PWM signal     */

            g_dachannel[4] = value;             /* turn operation mode ON       */
            TRDGRC1 = conv_TRD1( value );       /* set high time to TRD1        */
            PM1 &= 0xFD;                        /* set port to output           */
            P1 &= 0xFD;                         /* set port to low              */

            break;


        case 10:                                /* use D10 pin as PWM signal    */

            g_dachannel[5] = value;             /* turn operation mode ON       */
            TRDGRC1 = conv_TRD1( value );       /* set high time to TRD1        */
            PM1 &= 0xFB;                        /* set port to output           */
            P1 &= 0xFB;                         /* set port to low              */

            break;


        case 11:                                /* use D11 pin as PWM signal    */

                /* set new value to buffer  */
            g_dachannel[2] = value;             /* turn operation mode ON       */

            PM5 &= D11_OFF;                     /* set port to output           */

            break;


        default:                                /* otherwise are not allowed    */

            return( 0xFF );                     /* exit with error              */

    }   /* end of switch(pin)   */

    return(0x00);                               /* normal exit                  */

}   /* end of analogWrite function  */


/*******************************************************************************
* Function Name: conv_TRD1
* Description  : This function convert duty to TDR1 set data
* Arguments    : value
* Return Value : TDR1 set data
********************************************************************************/
uint16_t conv_TRD1(uint8_t value)
{

    uint16_t work16;

    switch ( value )
    {
        case 0:                                 /* PWM duty is 0%               */

            work16 = TRDGRA1;                   /* same as period data          */
            break;

        case 255:                               /* PWM duty is 100%             */

            work16 = ( TRDGRA1 + 1 );           /* more than period data        */
            break;

        default:                                /* otherwise are not allowed    */

            work16 = ( value - 1 );             /* set high level time-1        */

    }

    return( work16 );

}   /* end of conv_TRD1 */


/*******************************************************************************
* Function Name: anlogReference
* Description  : This function set refererence of A/D
* Arguments    : type
*               DEFAULT : VDDe
*               INTERNAL: internal reference voltage( 1.45V)
*               EXTERNAL: AREF pin
* Return Value : none
********************************************************************************/
void analogReference(uint8_t type)
{

    switch ( type )
    {
        case INTERNAL:
            ADM2 = 0x80;                        /* select internal voltage  */
            break;

        case EXTERNAL:
            ADM2 = 0x40;                        /* select AREF pin          */

            break;

        default:
            ADM2 = 0x00;                        /* select VDD for reverence */

    }

}   /* end of analogReferrence function */


/*******************************************************************************
* Function Name: r_tau0_channel0_interrupt
* Description  : This function is 4micro seconds interrupt routine
* Arguments    : none
* Return Value : none
********************************************************************************/
#ifdef   CS_ENV
static void __near r_tau0_channel0_interrupt(void)
#endif
#ifdef  IAR_ENV
#pragma vector = INTTM00_vect
__interrupt void r_tau0_channel0_interrupt(void)
#endif
{

    if ( 0xFE == g_times )
    {

/*----------------------------------------------

    PWM interval end and activate signal

------------------------------------------------*/
        g_da_time[0] = g_dachannel[0];          /* copy new data for D3 pin     */
        g_da_time[1] = g_dachannel[1];          /* copy new data for D5 pin     */
        g_da_time[2] = g_dachannel[2];          /* copy new data for D11 pin    */
        
        if ( 0 != g_da_time[0] )
        {
            D3PORT |= D3_ON;                    /* turn on software PWM1        */
        }
        else
        {
            D3PORT &= ~D3_ON;                   /* turn off software PWM1       */
        }

        if ( 0 != g_da_time[1] )
        {
            D5PORT |= D5_ON;                    /* turn on software PWM2        */
        }
        else
        {
            D5PORT &= ~D5_ON;                   /* turn off software PWM2       */
        }

        if ( 0 != g_da_time[2] )
        {
            D11PORT |= D11_ON;                  /* turn on software PWM3        */
        }
        else
        {
            D11PORT &= ~D11_ON;                 /* turn off software PWM3       */
        }

        g_times = 0x00;                         /* clear interval counter       */

    }
    else
    {

/*----------------------------------------------

    PWM 0-0xFD timing

------------------------------------------------*/

        g_times++;                              /* count up timing counter      */

        if ( g_times == g_da_time[0] )
        {
             D3PORT &= ~D3_ON;                  /* turn off software PWM1       */
        }

        if ( g_times == g_da_time[1] )
        {
             D5PORT &= ~D5_ON;                  /* turn off software PWM2       */
        }

        if ( g_times == g_da_time[2] )
        {
             D11PORT &= ~D11_ON;                /* turn off software PWM3       */
        }

    }

}   /* end of r_tau0_channel0_interrupt */
