/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/*******************************************************************************
* File Name    : LCD_LIB.c
* Version      : Rev. 1.00
* Device(s)    : R5F104ML
* Tool-Chain   : CCRL
* Description  : This file implements LCD control function via I2C bus
* Creation Date: 2021/02/01
* Modified Date: 2021/06/25
********************************************************************************/

/*******************************************************************************
Includes
********************************************************************************/
#include "r_cg_macrodriver.h"
#include "LCD_LIB.h"
#include "AR_LIB_TIME.h"
#include "AR_LIB_WIRE.h"
#include "r_cg_userdefine.h"
#include <stdarg.h>
#include <stdio.h>
#include "AR_WIRE_DEF.h"


/*******************************************************************************
Pragma directive
********************************************************************************/


/*******************************************************************************
Global variables and functions
********************************************************************************/
volatile unsigned char  g_lcd_command[2];   // for set command for LCD
volatile unsigned char  g_lcd_data[2];      // for set data for LCD



void set_2digit( uint8_t data);             // display 2digit data
void set_1digit( uint8_t data);             // display 1digit data
uint8_t set_command( uint8_t lcd_command);  // set command to LCD

//--------------------------------------------------------------
//
//    definition of wire API functions
//
//---------------------------------------------------------------
//API_Wire Wire = {
//    Wire_begin,                             // start of wire function as master
//    Wire_beginTransmission,                 // start transmit to slave
//    Wire_writeb,                            // data set to transmit buffer
//    Wire_writec,                            // data set to transmit buffer
//    Wire_endTransmission,                   // end transmit to slave
//    Wire_requestFrom,                       // start receiving
//    Wire_requestFromS,                      // start receiving
//    Wire_available,                         // check receive buffer
//    Wire_read,                              // read received data from buffer
//};

extern API_Wire Wire;
/*******************************************************************************
LCD display control functions for G14_FPB
********************************************************************************/

/*---------------------------------------------------------------------
* Function Name: init_LCD
* Description  : This function initialize LCD
* Arguments    : none
* Return Value : result
*                   0 : success
*                   2 : No_slave
*                   3 : No ACK for data
----------------------------------------------------------------------*/
 uint8_t init_LCD(void)
{

    uint8_t status;

    status =  set_command(CLRDISP);     // initialize LCD display
    if ( 0 != status ) return(2);

    delay(3);                           // wait for 3ms
    set_command(LCD_Mode);              // set LCD operation mode
    delayMicroseconds(60);              // wait for 60us
    set_command(DISPON);                // display on
    delayMicroseconds(60);              // wait for 60us
    set_command(ENTRY_Mode);            // set data entry mode
    delayMicroseconds(60);              // wait for 60us

    return(0);

}   /* end of init_LCD function   */


/*******************************************************************************
* Function Name: move_cursor
* Description  : This function move cursor position
* Arguments    : cursor address
*               col: 0 to 0x0F
*               row : 0(line1) /1(line2)
* Return Value : none
********************************************************************************/
void move_cursor( uint8_t col , uint8_t row)
{

    uint8_t work;

    work = (( row & 1 ) * 0x40);        // set top of line address
    work += ( col & 0x0F );             // add character position
    work |= 0x80;                       // get cursor move command

    set_command( work );                // set curso move command
    delayMicroseconds(60);              // wait for 60us

}   /* end of move_cursor function  */


/*******************************************************************************
* Function Name: print_LCD
* Description  : This function print 16chars 2lines
* Arguments    : data
* Return Value : none
********************************************************************************/
void print_LCD( uint8_t *point, uint8_t *point2)
{

    uint8_t data;
    uint8_t work;
    uint8_t *p_addr;

    p_addr = point;                         // set pointer
    move_cursor(0,0);                       // move cursor top of 1st line
    for ( work = 0 ; work < 16 ; ++work )
    {
        data = *p_addr;                     // read data
        if ( 0 == data )
        {
            data = 0x20;                    // change null data to space code
        }
        set_data( data );                   // send data 
        ++p_addr;
        delayMicroseconds(60);              // wait for 60us

    }

    p_addr = point2;                        // set pointer
    move_cursor(0,1);                       // move cursor top of 2nd line
    for ( work = 0 ; work < 16 ; ++work )
    {
        data = *p_addr;                     // read data
        if ( 0 == data )
        {
            data = 0x20;                    // change null data to space code
        }
        set_data( data );                   // send data 
        ++p_addr;
        delayMicroseconds(60);              // wait for 60us

    }

}   /* end of print_LCD function    */


/*******************************************************************************
* Function Name: set_2digitBCD
* Description  : This function display 2 digits
* Arguments    : display number(99-0)
* Return Value : none
********************************************************************************/
void set_2digitBCD( uint8_t data)
{

    uint8_t work;

    work = data;                        // get display data
    
    set_1digit((work / 10));            // display digit of 10
    delayMicroseconds(60);              // wait for 60us
    set_1digit((work % 10));            // display digit of 1
    delayMicroseconds(60);              // wait for 60us

}   /* end of set_2digitBCD function    */



/*******************************************************************************
* Function Name: set_2digit
* Description  : This function display 2 digits
* Arguments    : display number(packed BCD or HEX)
* Return Value : none
********************************************************************************/
void set_2digit( uint8_t data)
{

    uint8_t work;

    work = data;                        // get display data

    set_1digit( work >> 4 );            // display higher 4bit
    delayMicroseconds(60);              // wait for 60us
    set_1digit( data );                 // display lowwer 4bit
    delayMicroseconds(60);              // wait for 60us

}   /* end of set_2digit function   */


/*******************************************************************************
* Function Name: set_1digit
* Description  : This function display 1 digit(lowwer 4bit of data)
* Arguments    : display lowwer 4bit as ASCII
* Return Value : none
********************************************************************************/
void set_1digit( uint8_t data)
{

    uint8_t work;

    work = (data & 0x0F);               // mask unused bit
    if ( work < 0x0A )
    {
        work += 0x30;                   // convert 0-9
    }
    else
    {
        work += 0x37;                   // convert A-F
    }

    set_data(work);                     // display 1digit

}   /* end of set_1digit function   */


/*---------------------------------------------------------------------
* Function Name: set_command
* Description  : This function set command to LCD
* Arguments    : command
* Return Value : result
*                   0 : success
*                   2 : No_slave
*                   3 : No ACK for data
----------------------------------------------------------------------*/
uint8_t set_command( uint8_t lcd_command)
{

    uint8_t result;                     // result of transmission

    g_lcd_command[0] = COMBYTE;         // set command of LCD
    g_lcd_command[1] = lcd_command;     // set instruction for LCD

    Wire.beginTransmission(SLADDR_LCD); // select LCD display module

    Wire.write( (uint8_t *__near)g_lcd_command, 2 );    // send command for LCD

    result = Wire.endTransmission(RELEASE);// execute transmission

    return(result);

}   /* end of set_command function  */



/*******************************************************************************
* Function Name: set_data
* Description  : This function send data to LCD
* Arguments    : data code
* Return Value : result
*                   0 : success
*                   2 : No_slave
*                   3 : No ACK for data
********************************************************************************/
uint8_t set_data(uint8_t datacode)
{

    unsigned char result;               // result of transmission

    g_lcd_data[0] = DATABYTE;           // set command for LCD
    g_lcd_data[1] = datacode;           // set data for LCD

    Wire.beginTransmission(SLADDR_LCD); // select LCD display module

    Wire.write( (uint8_t *__near)g_lcd_data, 2 );   // send data for LCD

    result = Wire.endTransmission(RELEASE);// execute transmission

    return(result);

}   /* end of set_data function */




